"""
xShootableGun.py

Author: Sirius (based on tldnVaporScope from CyanWorlds)
Same as vapor scope, but without a GUI.
Gun rotation is done by pressing up, down, left, right
"""


from Plasma import *
from PlasmaTypes import *
import PlasmaControlKeys
from math import *




actGun          = ptAttribActivator(1,"Gun Clickable (to switch to gun-view)")
CameraObj       = ptAttribSceneobject(2, "Aiming camera")
animTurret      = ptAttribAnimation(3, 'Anim: Turret', netForce=1)
animPitch       = ptAttribAnimation(4, 'Anim: Pitch', netForce=1)
actFire         = ptAttribActivator(5,"Gun Trigger (to fire the gun)")
respShootGun    = ptAttribResponder(6, "Responder: Shoot", netForce=1)
SDLGunActivated = ptAttribString(7, "SDL name - is gun enabled ?")
actStopAtTop    = ptAttribActivator(8, "Actvtr: Stop Scope @ Top")
actStopAtBottom = ptAttribActivator(9, "Actvtr: Stop Scope @ Btm")


modName = "xShootableGun:"

engaged = False
atTop = False
atBot = False




class xShootableGun(ptResponder):
    def __init__(self):
        ptResponder.__init__(self)
        self.id = 5513 # random ID for max. Change to whatever you want, doesn't matter once in the KI
        self.version = 1
    
    
    def __del__(self):
        if engaged:
            PtGetAgeSDL()[SDLGunActivated.value] = (1,)
    
    
    def OnServerInitComplete(self):
        # request SDL changes, and make sure the activator is enabled/disabled
        ageSDL = PtGetAgeSDL()
        ageSDL.setFlags(SDLGunActivated.value,1,1)
        ageSDL.sendToClients(SDLGunActivated.value)
        ageSDL.setNotify(self.key, SDLGunActivated.value, 0.0)
        
        if ageSDL[SDLGunActivated.value][0] or not len(PtGetPlayerList()):
            actGun.enable()
        else:
            print modName, "Disabled at start"
            actGun.disable()
        
        actFire.disable()


    def OnNotify(self, state, id, events):
        global atTop
        global atBot
        print modName, "got event:", state,id,events
        
        if (not PtWasLocallyNotified(self.key) or not PtFindAvatar(events) == PtGetLocalAvatar()):
            return
        
        if not state: return
        
        if id == actGun.id:
            print modName, "got click from avatar"
            PtSendKIMessage(25, "Got click")
            self.EngageGun()
        
        elif id == actFire.id:
            self.Fire()
        
        elif id == actStopAtTop.id:
            print modName, "Scope at top."
            atTop = True
            atBot = False
        elif id == actStopAtBot.id:
            print modName, "Scope at bot."
            atBot = True
            atTop = False
    
    
    def OnSDLNotify(self, VARname, SDLname, playerID, tag):
        ageSDL = PtGetAgeSDL()
        if VARname == SDLGunActivated.value:
            if ageSDL[SDLGunActivated.value][0]:
                actGun.enable()
            else:
                actGun.disable()
    
    
    def OnControlKeyEvent(self, controlKey, activeFlag):
        global atTop
        global atBot
        if (controlKey == PlasmaControlKeys.kKeyExitMode and activeFlag):
            self.DisengageGun()
        elif (controlKey == PlasmaControlKeys.kKeyMoveBackward):
            print "rotate down"
            if activeFlag:
                animPitch.value.backwards(1)
                animPitch.value.resume()
                if atTop:
                    atTop = False
            else:
                animPitch.value.stop()
        elif (controlKey == PlasmaControlKeys.kKeyMoveForward):
            print "rotate up"
            if activeFlag:
                animPitch.value.backwards(0)
                animPitch.value.resume()
                if atBot:
                    atBot = False
            else:
                animPitch.value.stop()
        elif (controlKey == PlasmaControlKeys.kKeyRotateLeft):
            print "rotate left"
            if activeFlag:
                animTurret.value.backwards(0)
                animTurret.value.resume()
            else:
                animTurret.value.stop()
        elif (controlKey == PlasmaControlKeys.kKeyRotateRight):
            print "rotate left"
            if activeFlag:
                animTurret.value.backwards(1)
                animTurret.value.resume()
            else:
                animTurret.value.stop()
        elif (controlKey == PlasmaControlKeys.kKeyJump and activeFlag):
            self.Fire()


    def AvatarPage(self, avObj, pageIn, lastOut):
        if pageIn:
            return
        try:
            av = PtGetLocalAvatar()
        except:
            if engaged:
                PtGetAgeSDL()[SDLGunActivated.value] = (1,)
        else:
            if (avObj == av and engaged):
                PtGetAgeSDL()[SDLGunActivated.value] = (1,)
                print modName, "gun operator paged out, reenabled gun."
    
    
    def EngageGun(self):
        global engaged
        PtGetAgeSDL()[SDLGunActivated.value] = (0,)
        
        print modName, "changing cams"
        PtSendKIMessage(25, "Changing cams")
        
        cam = ptCamera()
        cam.undoFirstPerson()
        cam.disableFirstPersonOverride()
        cam.save(CameraObj.sceneobject.getKey())
        
        PtDisableMovementKeys()
        PtGetLocalAvatar().draw.disable()
        
        actFire.enable()
        
        PtEnableControlKeyEvents(self.key)
        
        engaged = True
    
    
    def DisengageGun(self):
        global engaged
        PtGetAgeSDL()[SDLGunActivated.value] = (1,)
        
        PtDisableControlKeyEvents(self.key)
        
        print modName, "Quitting gun..."
        PtSendKIMessage(25, "Quitting gun")
        
        cam = ptCamera()
        cam.restore(CameraObj.sceneobject.getKey())
        cam.enableFirstPersonOverride()
        PtRecenterCamera()
        
        PtEnableMovementKeys()
        PtGetLocalAvatar().draw.enable()
        
        actFire.disable()
        
        #animTurret.playToTime(0)
        #animPitch.playToTime(0)
        
        engaged = False
    
    
    def Fire(self):
        print modName, "FIRE !"
        PtSendKIMessage(25, "FIRE !")
        
        if respShootGun.value: respShootGun.run(self.key)
        
        PtRequestLOSScreen(self.key, 42, 0.5, 0.5, 10000, PtLOSObjectType.kShootable, PtLOSReportType.kReportHitOrMiss) # why ID 42 ? Well, why not ?
    
    
    def OnLOSNotify(self, ID, noHitFlag, sceneobject, hitPoint, distance):
        print modName, "Got notify !"
        print "id", ID
        print "noHitFlag", noHitFlag
        print "sceneobject", sceneobject
        print "hitPoint", hitPoint
        print "distance", distance
        
        if noHitFlag:
            PtSendKIMessage(25, "Hit nothing (ID %s)" % ID)
        else:
            PtSendKIMessage(25, "ID %s: hit %s at %s feet (%s, %s, %s)"
                % (ID, sceneobject.getName(), distance, hitPoint.getX(), hitPoint.getY(), hitPoint.getZ()))
        
        
        # run any resp/python attached
        if sceneobject:
            pmlist = sceneobject.getPythonMods()
            if (len(pmlist) > 0):
                print modName, "...python mod list:"
                for pm in pmlist:
                    print modName, pm.getName()
                    if string.lower(pm.getName()).find("hitme") != -1:
                        print modName, "HIT !"
                        notify = ptNotify(self.key)
                        notify.clearReceivers()
                        notify.addReceiver(pm)
                        notify.setActivate(1.0)
                        notify.send()
            
            resplist = sceneobject.getResponders()
            if (len(resplist) > 0):
                print modName, "...responder list:"
                for resp in resplist:
                    print modName, resp.getName()
                    if string.lower(resp.getName()).find("hitme") != -1:
                        print modName, "HIT !"
                        atResp = ptAttribResponder(42)
                        atResp.__setvalue__(resp)
                        atResp.run(self.key)
        
        
        PtShootBulletFromScreen(self.key, 0.5, 0.5, 1.0, 10000)




### Python Glue ###

glue_cl = None
glue_inst = None
glue_params = None
glue_paramKeys = None
try:
    x = glue_verbose
except NameError:
    glue_verbose = 0

def glue_getClass():
    global glue_cl
    if (glue_cl == None):
        try:
            cl = eval(glue_name)
            if issubclass(cl, ptModifier):
                glue_cl = cl
            elif glue_verbose:
                print ('Class %s is not derived from modifier' % cl.__name__)
        except NameError:
            if glue_verbose:
                try:
                    print ('Could not find class %s' % glue_name)
                except NameError:
                    print 'Filename/classname not set!'
    return glue_cl


def glue_getInst():
    global glue_inst
    if (type(glue_inst) == type(None)):
        cl = glue_getClass()
        if (cl != None):
            glue_inst = cl()
    return glue_inst


def glue_delInst():
    global glue_inst
    global glue_cl
    global glue_paramKeys
    global glue_params
    if (type(glue_inst) != type(None)):
        del glue_inst
    glue_cl = None
    glue_params = None
    glue_paramKeys = None


def glue_getVersion():
    inst = glue_getInst()
    ver = inst.version
    glue_delInst()
    return ver


def glue_findAndAddAttribs(obj, glue_params):
    if isinstance(obj, ptAttribute):
        if glue_params.has_key(obj.id):
            if glue_verbose:
                print 'WARNING: Duplicate attribute ids!'
                print ('%s has id %d which is already defined in %s' %
                      (obj.name, obj.id, glue_params[obj.id].name))
        else:
            glue_params[obj.id] = obj
    elif type(obj) == type([]):
        for o in obj:
            glue_findAndAddAttribs(o, glue_params)
    elif type(obj) == type({}):
        for o in obj.values():
            glue_findAndAddAttribs(o, glue_params)
    elif type(obj) == type(()):
        for o in obj:
            glue_findAndAddAttribs(o, glue_params)


def glue_getParamDict():
    global glue_paramKeys
    global glue_params
    if type(glue_params) == type(None):
        glue_params = {}
        gd = globals()
        for obj in gd.values():
            glue_findAndAddAttribs(obj, glue_params)
        glue_paramKeys = glue_params.keys()
        glue_paramKeys.sort()
        glue_paramKeys.reverse()
    return glue_params


def glue_getClassName():
    cl = glue_getClass()
    if (cl != None):
        return cl.__name__
    if glue_verbose:
        print ('Class not found in %s.py' % glue_name)
    return None


def glue_getBlockID():
    inst = glue_getInst()
    if (inst != None):
        return inst.id
    if glue_verbose:
        print ('Instance could not be created in %s.py' % glue_name)
    return None


def glue_getNumParams():
    pd = glue_getParamDict()
    if (pd != None):
        return len(pd)
    if glue_verbose:
        print ('No attributes found in %s.py' % glue_name)
    return 0


def glue_getParam(number):
    pd = glue_getParamDict()
    if (pd != None):
        if type(glue_paramKeys) == type([]):
            if (number >= 0) and (number < len(glue_paramKeys)):
                return pd[glue_paramKeys[number]].getdef()
            else:
                print ('glue_getParam: Error! %d out of range of attribute list' % number)
        else:
            pl = pd.values()
            if (number >= 0) and (number < len(pl)):
                return pl[number].getdef()
            elif glue_verbose:
                print ('glue_getParam: Error! %d out of range of attribute list' % number)
    if glue_verbose:
        print 'GLUE: Attribute list error'
    return None


def glue_setParam(id, value):
    pd = glue_getParamDict()
    if (pd != None):
        if pd.has_key(id):
            try:
                pd[id].__setvalue__(value)
            except AttributeError:
                if isinstance(pd[id], ptAttributeList):
                    try:
                        if type(pd[id].value) != type([]):
                            pd[id].value = []
                    except AttributeError:
                        pd[id].value = []
                    pd[id].value.append(value)
                else:
                    pd[id].value = value
        elif glue_verbose:
            print "setParam: can't find id=", id
    else:
        print 'setParma: Something terribly has gone wrong. Head for the cover.'


def glue_isNamedAttribute(id):
    pd = glue_getParamDict()
    if (pd != None):
        try:
            if isinstance(pd[id], ptAttribNamedActivator):
                return 1
            if isinstance(pd[id], ptAttribNamedResponder):
                return 2
        except KeyError:
            if glue_verbose:
                print ('Could not find id=%d attribute' % id)
    return 0


def glue_isMultiModifier():
    inst = glue_getInst()
    if isinstance(inst, ptMultiModifier):
        return 1
    return 0


def glue_getVisInfo(number):
    pd = glue_getParamDict()
    if pd != None:
        if type(glue_paramKeys) == type([]):
            if (number >= 0) and (number < len(glue_paramKeys)):
                return pd[glue_paramKeys[number]].getVisInfo()
            else:
                print ('glue_getVisInfo: Error! %d out of range of attribute list' % number)
        else:
            pl = pd.values()
            if (number >= 0) and (number < len(pl)):
                return pl[number].getVisInfo()
            elif glue_verbose:
                print ('glue_getVisInfo: Error! %d out of range of attribute list' % number)
    if glue_verbose:
        print 'GLUE: Attribute list error'
    return None
